@@ -17,7 +17,7 @@
 from trac.db import Table, Column, Index
 
 # Database version identifier. Used for automatic upgrades.
-db_version = 19
+db_version = 20
 
 def __mkreports(reports):
     """Utility function used to create report data in same syntax as the
@@ -179,9 +179,9 @@ SELECT p.value AS __color__,
    time AS created,
    changetime AS _changetime, description AS _description,
    reporter AS _reporter
-  FROM ticket t, enum p
+  FROM ticket t
+  LEFT JOIN enum p ON p.name = t.priority AND p.type = 'priority'
   WHERE status IN ('new', 'assigned', 'reopened') 
-AND p.name = t.priority AND p.type = 'priority'
   ORDER BY p.value, milestone, t.type, time
 """ % owner),
 #----------------------------------------------------------------------------
@@ -201,13 +201,13 @@ SELECT p.value AS __color__,
    time AS created,
    changetime AS _changetime, description AS _description,
    reporter AS _reporter
-  FROM ticket t, enum p
+  FROM ticket t
+  LEFT JOIN enum p ON p.name = t.priority AND p.type = 'priority'
   WHERE status IN ('new', 'assigned', 'reopened') 
-AND p.name = t.priority AND p.type = 'priority'
   ORDER BY (version IS NULL),version, p.value, t.type, time
 """ % owner),
 #----------------------------------------------------------------------------
-('All Tickets by Milestone',
+('Active Tickets by Milestone',
 """
 This report shows how to color results by priority,
 while grouping results by milestone.
@@ -223,9 +223,9 @@ SELECT p.value AS __color__,
    time AS created,
    changetime AS _changetime, description AS _description,
    reporter AS _reporter
-  FROM ticket t, enum p
+  FROM ticket t
+  LEFT JOIN enum p ON p.name = t.priority AND p.type = 'priority'
   WHERE status IN ('new', 'assigned', 'reopened') 
-AND p.name = t.priority AND p.type = 'priority'
   ORDER BY (milestone IS NULL),milestone, p.value, t.type, time
 """ % (db.concat('milestone', "' Release'"), owner)),
 #----------------------------------------------------------------------------
@@ -240,9 +240,9 @@ SELECT p.value AS __color__,
    id AS ticket, summary, component, milestone, t.type AS type, time AS created,
    changetime AS _changetime, description AS _description,
    reporter AS _reporter
-  FROM ticket t,enum p
+  FROM ticket t
+  LEFT JOIN enum p ON p.name = t.priority AND p.type = 'priority'
   WHERE status = 'assigned'
-AND p.name=t.priority AND p.type='priority'
   ORDER BY owner, p.value, t.type, time
 """),
 #----------------------------------------------------------------------------
@@ -257,9 +257,9 @@ SELECT p.value AS __color__,
    id AS ticket, summary, component, milestone, t.type AS type, time AS created,
    description AS _description_,
    changetime AS _changetime, reporter AS _reporter
-  FROM ticket t, enum p
+  FROM ticket t
+  LEFT JOIN enum p ON p.name = t.priority AND p.type = 'priority'
   WHERE status = 'assigned'
-AND p.name = t.priority AND p.type = 'priority'
   ORDER BY owner, p.value, t.type, time
 """),
 #----------------------------------------------------------------------------
@@ -273,14 +273,14 @@ SELECT p.value AS __color__,
    (CASE status 
       WHEN 'closed' THEN 'color: #777; background: #ddd; border-color: #ccc;'
       ELSE 
-        (CASE owner WHEN '$USER' THEN 'font-weight: bold' END)
+        (CASE owner WHEN $USER THEN 'font-weight: bold' END)
     END) AS __style__,
    id AS ticket, summary, component, status, 
    resolution,version, t.type AS type, priority, owner,
    changetime AS modified,
    time AS _time,reporter AS _reporter
-  FROM ticket t,enum p
-  WHERE p.name=t.priority AND p.type='priority'
+  FROM ticket t
+  LEFT JOIN enum p ON p.name = t.priority AND p.type = 'priority'
   ORDER BY (milestone IS NULL), milestone DESC, (status = 'closed'), 
         (CASE status WHEN 'closed' THEN modified ELSE (-1)*p.value END) DESC
 """),
@@ -298,9 +298,9 @@ SELECT p.value AS __color__,
    t.type AS type, priority, time AS created,
    changetime AS _changetime, description AS _description,
    reporter AS _reporter
-  FROM ticket t, enum p
-  WHERE t.status IN ('new', 'assigned', 'reopened') 
-AND p.name = t.priority AND p.type = 'priority' AND owner = '$USER'
+  FROM ticket t
+  LEFT JOIN enum p ON p.name = t.priority AND p.type = 'priority'
+  WHERE t.status IN ('new', 'assigned', 'reopened') AND owner = $USER
   ORDER BY (status = 'assigned') DESC, p.value, milestone, t.type, time
 """),
 #----------------------------------------------------------------------------
@@ -312,7 +312,7 @@ AND p.name = t.priority AND p.type = 'priority' AND owner = '$USER'
 """
 SELECT p.value AS __color__,
    (CASE owner 
-     WHEN '$USER' THEN 'My Tickets' 
+     WHEN $USER THEN 'My Tickets' 
      ELSE 'Active Tickets' 
     END) AS __group__,
    id AS ticket, summary, component, version, milestone, t.type AS type, 
@@ -320,10 +320,10 @@ SELECT p.value AS __color__,
    time AS created,
    changetime AS _changetime, description AS _description,
    reporter AS _reporter
-  FROM ticket t, enum p
+  FROM ticket t
+  LEFT JOIN enum p ON p.name = t.priority AND p.type = 'priority'
   WHERE status IN ('new', 'assigned', 'reopened') 
-AND p.name = t.priority AND p.type = 'priority'
-  ORDER BY (owner = '$USER') DESC, p.value, milestone, t.type, time
+  ORDER BY (owner = $USER) DESC, p.value, milestone, t.type, time
 """ % owner))
 
 
@@ -371,14 +371,14 @@ def get_data(db):
                (('anonymous', 'LOG_VIEW'),
                 ('anonymous', 'FILE_VIEW'),
                 ('anonymous', 'WIKI_VIEW'),
-                ('anonymous', 'WIKI_CREATE'),
-                ('anonymous', 'WIKI_MODIFY'),
+                ('authenticated', 'WIKI_CREATE'),
+                ('authenticated', 'WIKI_MODIFY'),
                 ('anonymous', 'SEARCH_VIEW'),
                 ('anonymous', 'REPORT_VIEW'),
                 ('anonymous', 'REPORT_SQL_VIEW'),
                 ('anonymous', 'TICKET_VIEW'),
-                ('anonymous', 'TICKET_CREATE'),
-                ('anonymous', 'TICKET_MODIFY'),
+                ('authenticated', 'TICKET_CREATE'),
+                ('authenticated', 'TICKET_MODIFY'),
                 ('anonymous', 'BROWSER_VIEW'),
                 ('anonymous', 'TIMELINE_VIEW'),
                 ('anonymous', 'CHANGESET_VIEW'),
@@ -386,7 +386,8 @@ def get_data(db):
                 ('anonymous', 'MILESTONE_VIEW'))),
            ('system',
              ('name', 'value'),
-               (('database_version', str(db_version)),)),
+               (('database_version', str(db_version)),
+                ('youngest_rev', ''))),
            ('report',
              ('author', 'title', 'query', 'description'),
                __mkreports(get_reports(db))))
