@@ -61,6 +61,17 @@ class TicketModuleBase(Component):
                                             'the %s field.' % (value, name))
                 elif not field.get('optional', False):
                     raise InvalidTicket('field %s must be set' % name)
+
+        try:
+            # comment index must be a number
+            int(req.args.get('cnum') or 0)
+            # replyto must be 'description' or a number
+            replyto = req.args.get('replyto')
+            if replyto != 'description':
+                int(replyto or 0)
+        except ValueError:
+            raise InvalidTicket('Invalid comment threading identifier')
+
         # Custom validation rules
         for manipulator in self.ticket_manipulators:
             for field, message in manipulator.validate_ticket(req, ticket):
@@ -353,7 +364,7 @@ class TicketModule(TicketModuleBase):
 
         href = format == 'rss' and req.abs_href or req.href
 
-        def produce((id, t, author, type, summary), status, fields,
+        def produce((id, t, author, type_, summary), status, fields,
                     comment, cid):
             if status == 'edit':
                 if 'ticket_details' in filters:
@@ -375,10 +386,11 @@ class TicketModule(TicketModuleBase):
             kind, verb = status_map[status]
             if format == 'rss':
                 title = 'Ticket #%s (%s %s): %s' % \
-                        (id, type.lower(), verb, summary)
+                        (id, type_ and type_.lower() or '', verb, summary)
             else:
-                title = Markup('Ticket <em title="%s">#%s</em> (%s) %s by %s',
-                               summary, id, type, verb, author)
+                typeinfo = type_ and '(%s)' % type_ or ''
+                title = Markup('Ticket <em title="%s">#%s</em> %s %s by %s',
+                               summary, id, typeinfo, verb, author)
             ticket_href = href.ticket(id)
             if cid:
                 ticket_href += '#comment:' + cid
@@ -573,6 +585,10 @@ class TicketModule(TicketModuleBase):
             if field['type'] in ('radio', 'select'):
                 value = ticket.values.get(field['name'])
                 options = field['options']
+                if field['name'] == 'milestone' \
+                    and not req.perm.has_permission('TICKET_ADMIN'):
+                    options = [opt for opt in options if not
+                               Milestone(self.env, opt, db=db).is_completed]
                 if value and not value in options:
                     # Current ticket value must be visible even if its not in the
                     # possible values
