@@ -45,7 +45,7 @@ class HTTPException(Exception):
         if args:
             self.message = self.message % args
         Exception.__init__(self, '%s %s (%s)' % (self.code, self.reason,
-                                                 message))
+                                                 self.message))
         return self
 
     def __str__(self):
@@ -326,9 +326,9 @@ class Request(object):
         self.send_response(status)
         if not url.startswith('http://') and not url.startswith('https://'):
             # Make sure the URL is absolute
-            url = urlparse.urlunparse((self.scheme,
-                                       urlparse.urlparse(self.base_url)[1],
-                                       url, None, None, None))
+            scheme, host = urlparse.urlparse(self.base_url)[:2]
+            url = urlparse.urlunparse((scheme, host, url, None, None, None))
+
         self.send_header('Location', url)
         self.send_header('Content-Type', 'text/plain')
         self.send_header('Pragma', 'no-cache')
