@@ -136,7 +136,7 @@ class RequestDispatcher(Component):
                                       IRequestHandler, 'WikiModule',
         """Name of the component that handles requests to the base URL.
         
-        Options include `TimeLineModule`, `RoadmapModule`, `BrowserModule`,
+        Options include `TimelineModule`, `RoadmapModule`, `BrowserModule`,
         `QueryModule`, `ReportModule` and `NewticketModule` (''since 0.9'').""")
 
     # Public API
@@ -164,6 +164,8 @@ class RequestDispatcher(Component):
         # Select the component that should handle the request
         chosen_handler = None
         early_error = None
+        req.authname = 'anonymous'
+        req.perm = NoPermissionCache()
         try:
             if not req.path_info or req.path_info == '/':
                 chosen_handler = self.default_handler
@@ -173,6 +175,19 @@ class RequestDispatcher(Component):
                         chosen_handler = handler
                         break
 
+            # Attach user information to the request early, so that
+            # the IRequestFilter can see it while preprocessing
+            if not getattr(chosen_handler, 'anonymous_request', False):
+                try:
+                    req.authname = self.authenticate(req)
+                    req.perm = PermissionCache(self.env, req.authname)
+                    req.session = Session(self.env, req)
+                    req.form_token = self._get_form_token(req)
+                except:
+                    req.authname = 'anonymous'
+                    req.perm = NoPermissionCache()
+                    early_error = sys.exc_info()
+
             chosen_handler = self._pre_process_request(req, chosen_handler)
         except:
             early_error = sys.exc_info()
@@ -182,22 +197,6 @@ class RequestDispatcher(Component):
                                         req.path_info),
                            None, None)
 
-        # Attach user information to the request
-        anonymous_request = getattr(chosen_handler, 'anonymous_request',
-                                    False)
-        if not anonymous_request:
-            try:
-                req.authname = self.authenticate(req)
-                req.perm = PermissionCache(self.env, req.authname)
-                req.session = Session(self.env, req)
-                req.form_token = self._get_form_token(req)
-            except:
-                anonymous_request = True
-                early_error = sys.exc_info()
-        if anonymous_request:
-            req.authname = 'anonymous'
-            req.perm = NoPermissionCache()
-
         # Prepare HDF for the clearsilver template
         try:
             use_template = getattr(chosen_handler, 'use_template', True)
@@ -295,6 +294,17 @@ def dispatch_request(environ, start_response):
     @param environ: the WSGI environment dict
     @param start_response: the WSGI callback for starting the response
     """
+
+    # SCRIPT_URL is an Apache var containing the URL before URL rewriting
+    # has been applied, so we can use it to reconstruct logical SCRIPT_NAME
+    script_url = environ.get('SCRIPT_URL')
+    if script_url is not None:
+        path_info = environ.get('PATH_INFO')
+        if not path_info:
+            environ['SCRIPT_NAME'] = script_url
+        elif script_url.endswith(path_info):
+            environ['SCRIPT_NAME'] = script_url[:-len(path_info)]
+
     if 'mod_python.options' in environ:
         options = environ['mod_python.options']
         environ.setdefault('trac.env_path', options.get('TracEnv'))
@@ -374,13 +384,22 @@ def dispatch_request(environ, start_response):
                                'options "TracEnv" or "TracEnvParentDir" are '
                                'missing. Trac requires one of these options '
                                'to locate the Trac environment(s).')
-    env = _open_environment(env_path, run_once=environ['wsgi.run_once'])
+    run_once = environ['wsgi.run_once']
 
-    if env.base_url:
-        environ['trac.base_url'] = env.base_url
+    env = env_error = None
+    try:
+        env = _open_environment(env_path, run_once=run_once)
+        if env.base_url_for_redirect:
+            environ['trac.base_url'] = env.base_url
+    except TracError, e:
+        env_error = e
 
     req = Request(environ, start_response)
     try:
+        if not env and env_error:
+            from trac.config import default_dir            
+            req.hdf = HDFWrapper([default_dir('templates')])
+            raise HTTPInternalError(env_error.message)
         try:
             try:
                 dispatcher = RequestDispatcher(env)
@@ -389,11 +408,12 @@ def dispatch_request(environ, start_response):
                 pass
             return req._response or []
         finally:
-            if environ.get('wsgi.multithread', False):
+            if not run_once:
                 env.shutdown(threading._get_ident())
 
     except HTTPException, e:
-        env.log.warn(e)
+        if env:
+            env.log.warn(e)
         if req.hdf:
             req.hdf['title'] = e.reason or 'Error'
             req.hdf['error'] = {
